@@ -41,7 +41,7 @@ import SCons.Defaults
 import SCons.Tool
 import SCons.Util
 
-mingw_paths = [
+mingw_base_paths = [
     r'c:\MinGW\bin',
     r'C:\cygwin64\bin',
     r'C:\msys64',
@@ -127,17 +127,24 @@ def find_version_specific_mingw_paths():
     One example of default mingw install paths is:
     C:\mingw-w64\x86_64-6.3.0-posix-seh-rt_v5-rev2\mingw64\bin
 
-    Use glob'ing to find such and add to mingw_paths
+    Use glob'ing to find such and add to mingw_base_paths
     """
     new_paths = glob.glob(r"C:\mingw-w64\*\mingw64\bin")
 
     return new_paths
 
 
+_mingw_all_paths = None
+
+def get_mingw_paths():
+    global _mingw_all_paths
+    if _mingw_all_paths is None:
+        _mingw_all_paths = mingw_base_paths + find_version_specific_mingw_paths()
+    return _mingw_all_paths
+
 def generate(env):
-    global mingw_paths
     # Check for reasoanble mingw default paths
-    mingw_paths += find_version_specific_mingw_paths()
+    mingw_paths = get_mingw_paths()
 
     mingw = SCons.Tool.find_program_path(env, key_program, default_paths=mingw_paths)
     if mingw:
@@ -204,6 +211,7 @@ def generate(env):
 
 
 def exists(env):
+    mingw_paths = get_mingw_paths()
     mingw = SCons.Tool.find_program_path(env, key_program, default_paths=mingw_paths)
     if mingw:
         mingw_bin_dir = os.path.dirname(mingw)
